#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <sched.h>
#include <dlfcn.h>
#include <errno.h>

#ifndef SYS_getcpu
#ifndef __NR_getcpu
#define SYS_getcpu 0
#else
#define SYS_getcpu __NR_getcpu
#endif
#endif

static int (*getcpup)(void);
static int (*getcpu_vdsop)(unsigned int *, unsigned int *, void *);

int getcpu_syscall (void)
{
	printf("calling getcpu via syscall\n");
	return syscall(SYS_getcpu);
}

int getcpu_vdso (void)
{
	int cpu, node;
	int ret;

	printf("calling getcpu from VDSO\n");
	ret = getcpu_vdsop(&cpu, &node, NULL);
	if (ret)
		return -1;
	return cpu;
}	

int setup(void)
{
	void *handle = dlopen("linux-vdso.so.1", RTLD_LAZY);

	getcpu_vdsop = NULL;
	if (handle) {
		getcpu_vdsop = dlsym(handle, "getcpu");
		dlclose(handle);
		if (getcpu_vdsop) {
			getcpup = getcpu_vdso;
			return 0;
		}
	}
	getcpup = dlsym(NULL, "sched_getcpu");
	if (!getcpup) {
		getcpup = dlsym(NULL, "getcpu");
		if (!getcpup)
			getcpup = getcpu_syscall;
	}
	return 0;
}
int
main(int argc, char **argv)
{
	int ret;
	cpu_set_t  set;

	setup();

	/* set our affinity */
	CPU_ZERO(&set);
	CPU_SET(1, &set);
	ret = sched_setaffinity(0, sizeof(cpu_set_t), &set);
	if (ret) {
		fprintf(stderr, "error setting affinity: %s",
			strerror(errno));
		exit(-1);
	}

	printf("cpu: %d\n", getcpup());
}
	
